from distutils.core import setup
setup(
  name = 'satoyama',
  packages = ['satoyama'], # this must be the same as the name above
  version = '0.1.1',
  description = 'Datamodels and utilities for sensor networks',
  author = 'Halfdan Rump',
  author_email = 'dalekk@gmail.com',
  url = 'https://github.com/DgFutureLab/satoyama', # use the URL to the github repo
  download_url = 'https://github.com/DgFutureLab/satoyama/tarball/0.1', # I'll explain this in a second
  keywords = ['sensor network', 'models'], # arbitrary keywords
  classifiers = [],
)
